from django.http.response import JsonResponse
from django.utils.translation import ugettext_lazy as _
from rest_framework import status

from jwt_devices import views
from jwt_devices.settings import api_settings


class PermittedHeadersMiddleware(object):
    """
    Middleware used to disallow sending the permanent_token header in other requests than during permanent token
    refresh to make sure naive FE developers do not send the fragile permanent token with each request.
    """

    def __init__(self, get_response=None):
        self.get_response = get_response

    def __call__(self, request):
        if self.get_response:
            return self.get_response(request)

    def process_view(self, request, view_func, view_args, view_kwargs):
        view_cls = getattr(view_func, "cls", None)
        if (view_cls and api_settings.JWT_PERMANENT_TOKEN_AUTH and request.META.get("HTTP_PERMANENT_TOKEN") and view_cls != views.DeviceRefreshJSONWebToken):
            return JsonResponse({
                "HTTP_PERMANENT_TOKEN": {
                    "details": _("Using the Permanent-Token header is disallowed for {}").format(type(view_cls))
                }
            }, status=status.HTTP_400_BAD_REQUEST)



import binascii
import os
import uuid

from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _


class Device(models.Model):
    """
    Device model used for permanent token authentication
    """
    permanent_token = models.CharField(max_length=255, unique=True)
    jwt_secret = models.UUIDField(default=uuid.uuid4, editable=False)
    created = models.DateTimeField(auto_now_add=True)
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    name = models.CharField(_("Device name"), max_length=255)
    details = models.CharField(_("Device details"), max_length=255, blank=True)
    last_request_datetime = models.DateTimeField(auto_now=True)

    def save(self, *args, **kwargs):
        if not self.permanent_token:
            self.permanent_token = self.generate_key()

        return super(Device, self).save(*args, **kwargs)

    def generate_key(self):
        return binascii.hexlify(os.urandom(20)).decode()



from django.contrib.auth import authenticate, get_user_model
from django.utils import timezone
from django.utils.translation import ugettext as _
from rest_framework import serializers
from rest_framework_jwt.compat import Serializer
from rest_framework_jwt.serializers import JSONWebTokenSerializer as OriginalJSONWebTokenSerializer
from rest_framework_jwt.settings import api_settings as rfj_settings

from jwt_devices.models import Device
from jwt_devices.settings import api_settings
from jwt_devices.utils import get_device_details

User = get_user_model()

jwt_payload_handler = rfj_settings.JWT_PAYLOAD_HANDLER
jwt_encode_handler = rfj_settings.JWT_ENCODE_HANDLER
jwt_devices_payload_handler = api_settings.JWT_DEVICES_PAYLOAD_HANDLER
jwt_devices_encode_handler = api_settings.JWT_DEVICES_ENCODE_HANDLER


class JSONWebTokenSerializer(OriginalJSONWebTokenSerializer):
    """
    Serializer used to obtain permanent token.
    """
    def validate(self, attrs):
        credentials = {
            self.username_field: attrs.get(self.username_field),
            "password": attrs.get("password")
        }

        if all(credentials.values()):
            user = authenticate(**credentials)

            if user:
                if not user.is_active:
                    msg = _("User account is disabled.")
                    raise serializers.ValidationError(msg)

                data = {
                    "user": user
                }
                if api_settings.JWT_PERMANENT_TOKEN_AUTH:
                    device_name, device_details = get_device_details(self.context["request"].META)
                    device = Device.objects.create(
                        user=user, last_request_datetime=timezone.now(),
                        name=device_name, details=device_details)

                    data["token"] = jwt_devices_encode_handler(jwt_devices_payload_handler(user, device=device))
                    data["device"] = device
                else:
                    data["token"] = jwt_encode_handler(jwt_payload_handler(user))

                return data
            else:
                msg = _("Unable to log in with provided credentials.")
                raise serializers.ValidationError(msg)
        else:
            msg = _("Must include \"{username_field}\" and \"password\".")
            msg = msg.format(username_field=self.username_field)
            raise serializers.ValidationError(msg)


class DeviceSerializer(serializers.ModelSerializer):
    class Meta:
        model = Device
        fields = ["id", "created", "name", "details", "last_request_datetime"]


class DeviceTokenRefreshSerializer(Serializer):
    HTTP_PERMANENT_TOKEN = serializers.CharField(required=True)

    def _get_device(self, attrs):
        permanent_token = attrs["HTTP_PERMANENT_TOKEN"]
        try:
            device = Device.objects.get(permanent_token=permanent_token)
        except Device.DoesNotExist:
            raise serializers.ValidationError({"HTTP_PERMANENT_TOKEN": _("Invalid permanent_token value.")})

        now = timezone.now()
        if now > device.last_request_datetime + api_settings.JWT_PERMANENT_TOKEN_EXPIRATION_DELTA:
            device.delete()
            raise serializers.ValidationError({"HTTP_PERMANENT_TOKEN": _("Permanent token has expired.")})

        if now > device.last_request_datetime + api_settings.JWT_PERMANENT_TOKEN_EXPIRATION_ACCURACY:
            device.last_request_datetime = now
            device.save()

        return device

    def validate(self, attrs):
        device = self._get_device(attrs)
        payload = jwt_devices_payload_handler(device.user, device=device)
        return {
            "token": jwt_devices_encode_handler(payload),
            "user": device.user
        }



import datetime

from django.conf import settings
from rest_framework.settings import APISettings

USER_SETTINGS = getattr(settings, "JWT_DEVICES", None)

DEFAULTS = {
    "JWT_PERMANENT_TOKEN_AUTH": True,
    "JWT_PERMANENT_TOKEN_EXPIRATION_ACCURACY": datetime.timedelta(minutes=30),
    "JWT_PERMANENT_TOKEN_EXPIRATION_DELTA": datetime.timedelta(days=7),

    "JWT_DEVICES_RESPONSE_PAYLOAD_HANDLER":
    "jwt_devices.utils.jwt_devices_response_payload_handler",

    "JWT_DEVICES_PAYLOAD_HANDLER":
    "jwt_devices.utils.jwt_devices_payload_handler",

    "JWT_DEVICES_ENCODE_HANDLER":
    "jwt_devices.utils.jwt_devices_encode_handler",

    "JWT_DEVICES_DECODE_HANDLER":
    "jwt_devices.utils.jwt_devices_decode_handler",
}

IMPORT_STRINGS = (
    "JWT_DEVICES_RESPONSE_PAYLOAD_HANDLER",
    "JWT_DEVICES_PAYLOAD_HANDLER",
    "JWT_DEVICES_ENCODE_HANDLER",
    "JWT_DEVICES_DECODE_HANDLER",
)
# List of settings that may be in string import notation.

api_settings = APISettings(USER_SETTINGS, DEFAULTS, IMPORT_STRINGS)



from django.contrib import admin

from jwt_devices.models import Device


class DeviceAdmin(admin.ModelAdmin):
    model = Device
    list_display = ["id", "user", "name", "created", "last_request_datetime"]
    list_filter = ["id", "user", "name", "created", "last_request_datetime"]
    search_fields = ["user__email", "name", "details"]
    fields = ["id", "name", "details", "created", "last_request_datetime"]
    readonly_fields = fields

    def get_queryset(self, request):
        return super().get_queryset(request).prefetch_related("user")

    def has_add_permission(self, request):
        return False


admin.site.register(Device, DeviceAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-10-10 21:52
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('jwt_devices', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='device',
            name='permanent_token',
            field=models.CharField(max_length=255, unique=True),
        ),
    ]



# -*- coding: utf-8 -*-
__title__ = "Django Rest Framework JWT Devices"
__version__ = "1.2.2"
__author__ = "Michal Proszek"
__license__ = "MIT"
__copyright__ = "Copyright 2017-2018 Arabella"

# Version synonym
VERSION = __version__



from datetime import datetime

from django.utils.translation import ugettext_lazy as _
from rest_framework import mixins, status, viewsets
from rest_framework.exceptions import NotFound, ValidationError
from rest_framework.generics import DestroyAPIView, GenericAPIView
from rest_framework.permissions import AllowAny, IsAuthenticated
from rest_framework.response import Response
from rest_framework_jwt.settings import api_settings as rfj_settings
from rest_framework_jwt.views import ObtainJSONWebToken as OriginalObtainJSONWebToken

from jwt_devices.models import Device
from jwt_devices.serializers import DeviceSerializer, DeviceTokenRefreshSerializer, JSONWebTokenSerializer
from jwt_devices.settings import api_settings

jwt_response_payload_handler = rfj_settings.JWT_RESPONSE_PAYLOAD_HANDLER
jwt_devices_response_payload_handler = api_settings.JWT_DEVICES_RESPONSE_PAYLOAD_HANDLER


class ObtainJSONWebTokenAPIView(OriginalObtainJSONWebToken):
    """Obtain JWT token
    API view used to obtain a JWT token along with creating a new Device object and returning permanent token.
    """
    serializer_class = JSONWebTokenSerializer
    permission_classes = [AllowAny]

    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)

        if serializer.is_valid():
            user = serializer.object.get("user") or request.user
            token = serializer.object.get("token")
            device = serializer.object.get("device", None)
            kwargs = {}
            if device:
                kwargs.update(dict(permanent_token=device.permanent_token, device_id=device.id))

            if api_settings.JWT_PERMANENT_TOKEN_AUTH:
                response_data = jwt_devices_response_payload_handler(token, user, request, **kwargs)
            else:
                response_data = jwt_response_payload_handler(token, user, request)

            response = Response(response_data)
            if rfj_settings.JWT_AUTH_COOKIE:
                expiration = (datetime.utcnow() + rfj_settings.JWT_EXPIRATION_DELTA)
                response.set_cookie(rfj_settings.JWT_AUTH_COOKIE,
                                    token,
                                    expires=expiration,
                                    httponly=True)
            return response

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class DeviceRefreshJSONWebToken(GenericAPIView):
    """Refresh JWT token
    API View used to refresh JSON Web Token using permanent token.
    The DeviceRefreshJSONWebToken view requires the Permanent-Token header to be set in the request headers.
    """
    serializer_class = DeviceTokenRefreshSerializer
    permission_classes = [AllowAny]

    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.META)
        if serializer.is_valid(raise_exception=True):
            data = jwt_devices_response_payload_handler(request=request, **serializer.validated_data)
            return Response(data, status=status.HTTP_200_OK)


class DeviceLogout(DestroyAPIView):
    """Logout user by deleting Device.
    The DeviceLogout view requires the Device-Id header to be set in the request headers.
    """
    queryset = Device.objects.all()
    permission_classes = [IsAuthenticated]

    def get_object(self):
        try:
            return self.get_queryset().get(user=self.request.user, id=self.request.META["HTTP_DEVICE_ID"])
        except (KeyError, ValueError):
            raise ValidationError(_("Device-Id header must be present in the request headers."))
        except Device.DoesNotExist:
            raise NotFound(_("Device does not exist."))


class DeviceViewSet(mixins.ListModelMixin, mixins.DestroyModelMixin, viewsets.GenericViewSet):
    """
    Simple viewset to list and delete Device objects related to user.
    """
    queryset = Device.objects.all()
    serializer_class = DeviceSerializer
    permission_classes = [IsAuthenticated]

    def get_queryset(self):
        return self.queryset.filter(user=self.request.user)


obtain_jwt_token = ObtainJSONWebTokenAPIView.as_view()
device_refresh_token = DeviceRefreshJSONWebToken.as_view()
device_logout = DeviceLogout.as_view()



import jwt
from django.utils.translation import ugettext_lazy as _
from rest_framework.exceptions import NotFound
from rest_framework_jwt.settings import api_settings as rfj_settings

from jwt_devices.models import Device

jwt_payload_handler = rfj_settings.JWT_PAYLOAD_HANDLER
jwt_response_payload_handler = rfj_settings.JWT_RESPONSE_PAYLOAD_HANDLER


def jwt_devices_get_secret_key(payload=None):
    try:
        return Device.objects.get(pk=payload.get("device_id")).jwt_secret.hex
    except Device.DoesNotExist:
        raise NotFound(_("Permanent token has expired."))


def jwt_devices_payload_handler(user, device=None):
    payload = jwt_payload_handler(user)
    if device:
        payload["device_id"] = str(device.pk)
    return payload


def jwt_devices_encode_handler(payload):
    return jwt.encode(
        payload,
        jwt_devices_get_secret_key(payload),
        rfj_settings.JWT_ALGORITHM
    ).decode("utf-8")


def jwt_devices_response_payload_handler(token, user=None, request=None, **kwargs):
    """
    Returns the response data for both the login and refresh views.
    """
    data = jwt_response_payload_handler(token, user, request)
    permanent_token = kwargs.get("permanent_token")
    if permanent_token:
        data["permanent_token"] = permanent_token
    device_id = kwargs.get("device_id")
    if device_id:
        data["device_id"] = device_id

    return data


def jwt_devices_decode_handler(token):
    options = {
        "verify_exp": rfj_settings.JWT_VERIFY_EXPIRATION,
    }
    # get user from token, BEFORE verification, to get user secret key
    unverified_payload = jwt.decode(token, None, False)
    return jwt.decode(
        token,
        jwt_devices_get_secret_key(unverified_payload),
        rfj_settings.JWT_VERIFY,
        options=options,
        leeway=rfj_settings.JWT_LEEWAY,
        audience=rfj_settings.JWT_AUDIENCE,
        issuer=rfj_settings.JWT_ISSUER,
        algorithms=[rfj_settings.JWT_ALGORITHM]
    )


def get_device_details(headers):
    device_name = headers.get("HTTP_X_DEVICE_MODEL")
    user_agent = headers.get("HTTP_USER_AGENT", "")
    if not device_name:
        device_name = user_agent
        device_details = ""
    else:
        device_details = user_agent

    return device_name, device_details



# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-07-21 08:06
from __future__ import unicode_literals

import uuid

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Device',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('permanent_token', models.CharField(max_length=255, unique=True, serialize=False)),
                ('jwt_secret', models.UUIDField(default=uuid.uuid4, editable=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('name', models.CharField(max_length=255, verbose_name='Device name')),
                ('details', models.CharField(blank=True, max_length=255, verbose_name='Device details')),
                ('last_request_datetime', models.DateTimeField(auto_now=True)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



import jwt
from django.utils.translation import ugettext as _
from rest_framework import exceptions
from rest_framework_jwt.authentication import JSONWebTokenAuthentication
from rest_framework_jwt.settings import api_settings as rfj_settings

from jwt_devices.settings import api_settings

jwt_decode_handler = rfj_settings.JWT_DECODE_HANDLER
jwt_devices_decode_handler = api_settings.JWT_DEVICES_DECODE_HANDLER


class PermanentTokenAuthentication(JSONWebTokenAuthentication):
    def authenticate(self, request):
        jwt_value = self.get_jwt_value(request)
        if jwt_value is None:
            return None

        try:
            if api_settings.JWT_PERMANENT_TOKEN_AUTH:
                payload = jwt_devices_decode_handler(jwt_value)
            else:
                payload = jwt_decode_handler(jwt_value)
        except jwt.ExpiredSignature:
            msg = _("Signature has expired.")
            raise exceptions.AuthenticationFailed(msg)
        except jwt.DecodeError:
            msg = _("Error decoding signature.")
            raise exceptions.AuthenticationFailed(msg)
        except jwt.InvalidTokenError:
            raise exceptions.AuthenticationFailed()

        user = self.authenticate_credentials(payload)

        return user, jwt_value
